#ifndef _IP_NAT_H
#define _IP_NAT_H
#include "conntrack/ip_conntrack.h"

#define IP_NAT_MAPPING_TYPE_MAX_NAMELEN 16

enum ip_nat_manip_type
{
	IP_NAT_MANIP_SRC,
	IP_NAT_MANIP_DST
};

/* The manipulations for a specific connection. */
struct ip_nat_manipulation
{
	/* The type of manipulation to perform */
	enum ip_nat_manip_type maniptype;

	/* Replacement. */
	struct ip_conntrack_manip manip;
};

/* 2.0 firewalling went from 64 through 71 (and +256, +512, etc). */
/* 2.2 firewalling (+ masq) went from 64 through 76 */
/* 2.4 firewalling went 64 through 67. */
#define SO_ORIGINAL_DST 80

#define IP_NAT_RANGE_MAP_IPS 1
#define IP_NAT_RANGE_PROTO_SPECIFIED 2
/* Used internally by get_unique_tuple(). */
#define IP_NAT_RANGE_FULL 4

/* Single range specification. */
struct ip_nat_range
{
	/* Set to OR of flags above. */
	unsigned int flags;

	/* Inclusive: network order. */
	u_int32_t min_ip, max_ip;

	union ip_conntrack_manip_proto min, max;
};

#ifdef __KERNEL__
#include <linux/list.h>

/* Hashes for by-source and IP/protocol. */
struct ip_nat_hash
{
	struct list_head list;

	/* conntrack we're embedded in: NULL if not in hash. */
	struct ip_conntrack *conntrack;
};

/* At most two manipulations. */
#define IP_NAT_MAX_MANIPS 2

/* The structure embedded in the conntrack structure. */
struct ip_nat_info
{
	/* Set to zero when conntrack created. */
	int initialized;

	/* Indexed by conntrack direction. */
	struct {
		/* Which hook the manipulation happens on. */
		unsigned int hook;
		/* Number of manipulations. */
		unsigned int num_manips;
		/* Manipulations to occur at each conntrack in this dirn. */
		struct ip_nat_manipulation manipulation[IP_NAT_MAX_MANIPS];
	} dirinfo[2];

	/* The mapping type which created us (NULL for null mapping). */
	const struct ip_nat_mapping_type *mtype;

	struct ip_nat_hash bysource, byipsproto;

	/* Helper (NULL if none). */
	struct ip_nat_helper *helper;
};

/* A range consists of an array of 1 or more ip_nat_range */
struct ip_nat_multi_range
{
	unsigned int rangesize;

	/* hangs off end. */
	struct ip_nat_range range[1];
};

/* Is this tuple already taken? (not by us)*/
extern int ip_nat_used_tuple(const struct ip_conntrack_tuple *tuple,
			     const struct ip_conntrack *ignored_conntrack);

/* Calculate relative checksum. */
extern u_int16_t ip_nat_cheat_check(u_int32_t oldvalinv,
				    u_int32_t newval,
				    u_int16_t oldcheck);
#endif /*__KERNEL__*/
#endif
