#ifndef _IPTABLES_H
#define _IPTABLES_H

#include "libiptc/libiptc.h"

/* Include file for additions: new matches and targets. */
struct iptables_match
{
	struct iptables_match *next;

	ipt_chainlabel name;

	/* Function which prints out usage message. */
	void (*help)(void);

	/* Initialize the info union. */
	void (*init)(union ipt_matchinfo *info, unsigned int *nfcache);

	/* Function which parses command options; returns true if it
           ate an option */
	int (*parse)(int c, char **argv, int invert, unsigned int *flags,
		     struct ipt_entry *entry);

	/* Final check; exit if not ok. */
	void (*final_check)(unsigned int flags);

	/* Prints out the union ipt_matchinfo. */
	void (*print)(const union ipt_matchinfo *info, int numeric);

	/* Pointer to list of extra command-line options */
	struct option *extra_opts;
};

struct iptables_target
{
	struct iptables_target *next;

	ipt_chainlabel name;

	/* Function which prints out usage message. */
	void (*help)(void);

	/* Initialize the info union. */
	void (*init)(union ipt_targinfo *targinfo, unsigned int *nfcache);

	/* Function which parses command options; returns true if it
           ate an option */
	int (*parse)(int c, char **argv, int invert, unsigned int *flags,
		     struct ipt_entry *entry);

	/* Final check; exit if not ok. */
	void (*final_check)(unsigned int flags);

	/* Prints out the union ipt_targinfo. */
	void (*print)(const union ipt_targinfo *info, int numeric);

	/* Pointer to list of extra command-line options */
	struct option *extra_opts;
};

/* Your shared library should call one of these. */
extern void register_match(struct iptables_match *me);
extern void register_target(struct iptables_target *me);

/* Functions we share */
enum exittype {
	OTHER_PROBLEM = 1,
	PARAMETER_PROBLEM,
	VERSION_PROBLEM
};
extern void exit_printhelp() __attribute__((noreturn));
extern void exit_tryhelp(int) __attribute__((noreturn));
int check_inverse(const char option[], int *invert);
extern int string_to_number(const char *, int, int);
void exit_error(enum exittype, char *, ...)__attribute__((noreturn,
							  format(printf,2,3)));
#endif /*_IPTABLES_H*/
