/*
 * This is a module which is used for queueing packets.
 */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/init.h>
#include <net/icmp.h>
#include <net/udp.h>
#include <net/tcp.h>
#include "packet-filter/kernel/ip_tables.h"

#include "packet-filter/extentions/ipt_QUEUE.h"

/* Make init and cleanup non-static, so gcc doesn't warn about unused,
   but don't export the symbols */
EXPORT_NO_SYMBOLS;

static int ipt_queue_target(struct sk_buff *skb,
			    const char *indev,
			    const char *outdev,
			    const union ipt_targinfo *info,
			    unsigned int pos)
{
	struct ipt_queue_info *queueinfo = (void *)info->other;

	if (skb->nfreason == NF_REASON_SET_BY_IPCHAINS
	    || skb->nfreason == NF_REASON_NONE) {
		skb->nfreason = NF_REASON_SET_BY_IPCHAINS;
		skb->nfmark = queueinfo->mark;
	}

	return -NF_QUEUE - 1;
}

static int ipt_queue_checkentry(const union ipt_targinfo *info,
				unsigned int pos,
				unsigned int finalpos)
{
	return 1;
}

static struct ipt_target ipt_queue_reg
= { { NULL, NULL }, "QUEUE", ipt_queue_target, ipt_queue_checkentry, NULL, 
    THIS_MODULE };

int __init init(void)
{
    if (ipt_register_target(&ipt_queue_reg))
	return -EINVAL;

    return 0;
}

void __exit cleanup(void)
{
    ipt_unregister_target(&ipt_queue_reg);
}

module_init(init);
module_exit(cleanup);
