#include <stdlib.h>
#include <stdio.h>
#include <netinet/ip.h>
#include <sys/types.h>

static u_int16_t 
csum_partial(void *buffer, unsigned int len, u_int16_t prevsum)
{
	u_int32_t sum = 0;
	u_int16_t *ptr = buffer;

	while (len > 1)  {
		sum += *ptr++;
		len -= 2;
	}
	if (len) {
		union {
			u_int8_t byte;
			u_int16_t wyde;
		} odd;
		odd.wyde = 0;
		odd.byte = *((u_int8_t *)ptr);
		sum += odd.wyde;
	}
	sum = (sum >> 16) + (sum & 0xFFFF);
	sum += prevsum;
	return (sum + (sum >> 16));
}

void fillrandom(void *ptr, size_t size)
{
	while (size) {
		*((unsigned char *)ptr) = random();
		ptr++;
		size--;
	}
}

unsigned int i;

u_int16_t fast_ipcsum(u_int32_t oldsrc, u_int32_t olddst, 
		      u_int32_t src, u_int32_t dst,
		      u_int16_t oldsum)
{
	u_int32_t diffs[4] = { ~oldsrc, ~olddst, src, dst };
	return csum_partial(diffs, sizeof(diffs), oldsum);
}

int main()
{
	srandom(0);

	/* Pick random IP packets.  Calculate checksum.  Change IP address,
	 * figure checksum via the shortcut, then check it. */
	for (i = 0;; i++) {
		struct iphdr hdr;
		u_int32_t src, dst, oldsrc, olddst;
		u_int16_t oldsum, sum;
		
		fillrandom(&hdr, sizeof(hdr));

		hdr.check = 0;
		oldsum = csum_partial(&hdr, sizeof(hdr), 0);

		fillrandom(&src, sizeof(src));
		fillrandom(&dst, sizeof(dst));

		oldsrc = hdr.saddr;
		olddst = hdr.daddr;
		hdr.saddr = src;
		hdr.daddr = dst;

		sum = csum_partial(&hdr, sizeof(hdr), 0);
		if (fast_ipcsum(oldsrc, olddst, src, dst, oldsum) != sum)
			printf("FAIL %i: old 0x%08X, new 0x%08X, fast 0x%08X\n",
			       i, oldsum, sum, 
			       fast_ipcsum(oldsrc, olddst, src, dst, oldsum));
	}
	return 0;
}
