#! /bin/bash

# Need job control.
set -m

# Simple matching.  We can match a packet on (ignoring output interface):
# SRCIP(mask) DSTIP(mask) inif(mask) proto tos frag

#  Each mask can be partial, inverse partial, exact, inverse exact or
#  match-all. Proto, TOS and frag can be exact, inverse exact, or
#  match-all.

# So we have 5 * 5 * 5 * 3 * 3 * 3 = 3375 rules.

# For each mask, we have three packet settings: one which matches
# exactly, one which matches when masked, and one which never matches.
# For the other three, we have ones which match and ones which don't.

# This means 3 * 3 * 3 * 2 * 2 * 2 = 216 packets.

#     PACKET        RULE     
#  EXACT NEVER  ALL INV EXACT MATCH  NUMERIC
#      0     0    0   0    0      0    0
#      0     0    0   0    1      0    2
#      0     0    0   1    0      1    5
#      0     0    0   1    1      1    7
#      0     0    1   0    0      1    9
#      1     0    0   0    0      1   33
#      1     0    0   0    1      1   35
#      1     0    0   1    0      0   36
#      1     0    0   1    1      0   38
#      1     0    1   0    0      1   41
#      0     1    0   0    0      0   16
#      0     1    0   0    1      0   18
#      0     1    0   1    0      1   21
#      0     1    0   1    1      1   23
#      0     1    1   0    0      1   25
#
#  PACKET    RULE      MATCH   NUMERIC
#  IS      IS  ANY
#   0      0   0       1       1
#   1      0   0       0       8
#   0      1   0       0       4
#   1      1   0       1      13
#   0      0   1       1       3
#   1      0   1       1      11

FIVE_RULE="0 2 5 7 9 33 35 36 38 41 16 18 21 23 25"
THREE_RULE="1 8 4 13 3 11"

IPTABLES=../packet-filter/userspace/iptables

$IPCHAINS -N foo || exit 1
$IPCHAINS -A forward -i tap0 -j foo || exit 1

# Dummy rule to replace.
$IPCHAINS -A foo

# Disable ICMP redirect generation.
echo 0 > /proc/sys/net/ipv4/conf/tap0/send_redirects 

# Total expected matches: 9 * 9 * 9 * 4 * 4 * 4.
./rcv_ip 3 `expr 9 \* 9 \* 9 \* 4 \* 4 \* 4` > $TMPFILE < /dev/tap1 &
sleep 1

[ -n "$VERBOSE" ] && echo Sending packets...
for srcip in $FIVE_RULE; do
    for dstip in $FIVE_RULE; do
	for iface in $FIVE_RULE
	    for proto in $THREE_RULE; do
		for tos in $THREE_RULE; do
		    for frag in $THREE_RULE; do
			$IPCHAINS -R foo 1 ${R_SRC[$srcip]} ${R_DST[$dstip]} ${R_IF[$iface]} ${R_PROTO[$proto]} ${R_TOS[$tos]} ${R_FRAG[$frag]} || exit 1
			./gen_ip ${P_SRC[$srcip]} ${P_DST[$dstip]} ${P_PROTO[$proto]} ${P_TOS[$tos]} ${P_FRAG[$frag]} > ${P_IF[$iface]} || exit 1
		    done
		done
	    done
	done
    done
done

if wait %./rcv_ip
then :
else
    echo rcv_ip failed (`wc -l < $TMPFILE`) lines.
    exit 1
fi

[ -n "$VERBOSE" ] && echo Analysing results...

while read LINE
do
for srcip in $FIVE_RULE; do
    for dstip in $FIVE_RULE; do
	for iface in $FIVE_RULE
	    for proto in $THREE_RULE; do
		for tos in $THREE_RULE; do
		    for frag in $THREE_RULE; do
			# Lowest bit means match: all lowest bits
			# must be set for an overall match.
			if (( ($srcip | $dstip | $inif | $outif | $proto | $tos | $frag) & 0x01 ))
			then
			    if [ "$LINE" != "${P_SRC[$srcip]} ${P_DST[$dstip]} ${P_PROTO[$proto]}${P_TOS[$tos]}${P_FRAG[$frag]}" ]
			    then
				echo Line \`"$LINE"\' vs \`"${P_SRC[$srcip]} ${P_DST[$dstip]} ${P_PROTO[$proto]}${P_TOS[$tos]}${P_FRAG[$frag]}"\'
				exit 1
			    fi
			fi
		    done
		done
	    done
	done
    done
done < $TMPFILE

echo 1 > /proc/sys/net/ipv4/conf/tap0/send_redirects 
exit 0
